package com.nationrel.modules.rga.general.util.common;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;

import static java.lang.Double.NEGATIVE_INFINITY;
import static java.lang.Double.POSITIVE_INFINITY;

/**
 * 数字格式通用工具类
 *
 * @author ted
 */
public final class NumberUtil {

    /**
     * <p>
     * Title:
     * </p>
     * <p>
     * Description:
     * </p>
     */
    private NumberUtil() {
        super();
    }

    /**
     * @Title: format
     * @Description: 按指定位数四舍五入格式化数据
     * @param formatObj 待格式的数字
     * @param format    格式字符串
     * @return String
     */
    public static String format(final Object formatObj, final String format) {
        if (formatObj == null) {
            return "";
        }

        // 数字格式处理
        DecimalFormat df = new DecimalFormat(format);
        df.setRoundingMode(RoundingMode.HALF_UP);
        BigDecimal bdc = new BigDecimal(String.valueOf(formatObj));

        return df.format(bdc);
    }

    /**
     * 按四舍五入保留小数位数
     *
     * @param dbln     待处理数字
     * @param radixbit : 小数位数
     * @return 四舍五入的结果 0.04 0.05 0.06 -0.04 -0.05 -0.06 ROUND_CEILING 0.1 0.1 0.1
     *         0.0 0.0 0.0 ROUND_FLOOR 0.0 0.0 0.0 -0.1 -0.1 -0.1 ROUND_UP 0.1 0.1
     *         0.1 -0.1 -0.1 -0.1 ROUND_DOWN 0.0 0.0 0.0 0.0 0.0 0.0 ROUND_HALF_DOWN
     *         0.0 0.0 0.1 0.0 0.0 -0.1 ROUND_HALF_UP 0.0 0.1 0.1 0.0 -0.1 -0.1
     *         ROUND_HALF_EVEN 0.0 0.0 0.1 0.0 0.0 -0.1
     */
    public static BigDecimal roundx(Object dbln, int radixbit) {
        if (dbln == null) {
            return null;
        }
        BigDecimal bdc = new BigDecimal(String.valueOf(dbln));
        return bdc.setScale(radixbit, BigDecimal.ROUND_HALF_UP);
    }

    /**
     * 判断数值是否为空
     * @param value
     * @return
     */
    public static boolean isNull(Double value){
        return Double.isNaN(value) || value == POSITIVE_INFINITY || value == NEGATIVE_INFINITY;
    }
    /**
     * @Title: round
     * @Description: 双精度四舍五入处理
     * @param dbln     待处理数字
     * @param radixbit 小数位数
     * @return 四舍五入的结果
     */
    public static Double round(Double dbln, int radixbit) {
        if (dbln == null) {
            return null;
        }
        BigDecimal bdc = new BigDecimal(String.valueOf(dbln));
        return bdc.setScale(radixbit, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * @Title: ceil
     * @Description: 向正无穷舍入处理
     * @param dbln     待处理数字
     * @param radixbit 小数位数
     * @return 向正无穷舍入的结果
     */
    public static Double ceil(Double dbln, int radixbit) {
        if (dbln == null) {
            return null;
        }
        BigDecimal bdc = new BigDecimal(String.valueOf(dbln));
        return bdc.setScale(radixbit, BigDecimal.ROUND_UP).doubleValue();
    }

    /**
     * @Title: floor
     * @Description: 向负无穷舍入处理
     * @param dbln     待处理数字
     * @param radixbit 小数位数
     * @return 向负无穷舍入的结果
     */
    public static Double floor(Double dbln, int radixbit) {
        if (dbln == null) {
            return null;
        }
        BigDecimal bdc = new BigDecimal(String.valueOf(dbln));
        return bdc.setScale(radixbit, BigDecimal.ROUND_FLOOR).doubleValue();
    }

    /**
     * Object转换为BigDecimal
     *
     * @param objIn Object
     * @return 处理后的BigDecimal
     */
    public static BigDecimal parseBigDecimal(Object objIn) {
        if (objIn == null) {
            return null;
        } else {
            return new BigDecimal(String.valueOf(objIn));
        }
    }

    /**
     * @Title:
     * @Description: 对象转换为短整
     * @param obj 待转换参数
     * @return Short
     */
    public static Short parseShort(Object obj) {
        if (obj == null || "".equals(obj)) {
            return null;
        } else {
            return Short.valueOf(String.valueOf(obj));
        }
    }

    /**
     * @Title:
     * @Description: 对象转换为整型
     * @param obj 待转换参数
     * @return Integer
     */
    public static Integer parseInteger(Object obj) {
        if (obj == null || "".equals(obj)) {
            return null;
        } else {
            return Integer.valueOf(String.valueOf(obj));
        }
    }

    /**
     * @Title: parseDouble
     * @Description: 将科学计数法数字转换为double
     * @param str 待转换参数
     * @return String
     */
    public static Integer parseInteger(String str) {
        if (ValidateUtils.isNotEmpty(str) && str.matches("^-?(?!0\\d)\\d+(\\.\\d{1,})?([Ee]{0,1}[-]{0,1}\\d+)?$") || str.contains("E")) {
            BigDecimal big = new BigDecimal(str);
            return big.intValue();
        } else {
            return null;
        }
    }

    /**
     * @Title: parseDouble
     * @Description: 将科学计数法数字转换为double
     * @param str 待转换参数
     * @return String
     */
    public static Double parseDouble(String str) {
        if (ValidateUtils.isNotEmpty(str) && str.matches("^-?(?!0\\d)\\d+(\\.\\d{1,})?([Ee]{0,1}[-]{0,1}\\d+)?$")) {
            BigDecimal big = new BigDecimal(str);
            return big.doubleValue();
        } else {
            return null;
        }
    }

    /**
     * @Title: parseDouble
     * @Description: 将科学计数法数字转换为double
     * @param str 待转换参数
     * @return String
     */
    public static Double parseDouble1(String str) {
        if (ValidateUtils.isNotEmpty(str) && str.matches("^-?(?!0\\d)\\d+(\\.\\d{1,})?([Ee]{0,1}[-]{0,1}\\d+)?$")) {
            BigDecimal big = new BigDecimal(str);
            return big.doubleValue();
        } else {
            return 0.0;
        }
    }

    /**
     * @Title: formatDouble
     * @Description: 格式化Double为String
     * @param dbParam        待转化Double
     * @param fractionDigits 小数位
     * @return String
     */
    public static String formatDouble(Double dbParam, int fractionDigits) {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(fractionDigits);
        df.setMinimumFractionDigits(0);
        df.setGroupingUsed(false);
        df.setRoundingMode(RoundingMode.HALF_UP);
        return df.format(dbParam);
    }

    /**
     * @Title: formatBigDecimal
     * @Description: 格式化BigDecimal为String
     * @param bdcParam       待转化BigDecimal
     * @param fractionDigits 小数位
     * @return String
     */
    public static String formatBigDecimal(BigDecimal bdcParam, int fractionDigits) {
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(fractionDigits);
        df.setMinimumFractionDigits(0);
        df.setGroupingUsed(false);
        df.setRoundingMode(RoundingMode.HALF_UP);
        return df.format(bdcParam);
    }

    /**
     * @Title: log10
     * @Description: 求10的对数
     * @param dbObj 待计算参数
     * @return Double
     */
    public static Double log10(Double dbObj) {
        if (dbObj == null) {
            return null;
        }
        if (dbObj <= 0d) {
            return null;
        } else {
            return Math.log10(dbObj);
        }
    }

    /**
     * 将各种类型转换为double
     *
     * @param val
     * @return
     */
    public static Double convNumber(String val) {
        // 是否为数字
        if (ValidateUtils.isNumber(val)) {

            return Double.parseDouble(String.format("%.6f", Double.parseDouble(val)));
        } else if (ValidateUtils.isScientificNum(val)) {
            // 科学计数法
            return NumberUtil.round(NumberUtil.parseDouble(val), 4);
        } else {
            return 0.0;
        }
    }

    /**
     * 消暑取整
     * @param s
     * @return
     */
    public static double parseDecimal(double s) {
        DecimalFormat df = new DecimalFormat("0.0000");
        return Double.parseDouble(df.format(s));
    }
}
