package com.nationrel.modules.rga.handler;

import com.nationrel.modules.rga.general.common.ServerResponse;
import com.nationrel.modules.rga.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeLanguageKeyEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeProgramExceptionEnum;
import com.nationrel.modules.rga.general.exception.DescribeException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.ConstraintViolationException;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.core.annotation.Order;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;


/**
 * 创建人是: hcs 创建时间: 2018/2/7 0007.
 * 全局异常处理类
 * 该全局异常处理旨在搜集系统中出现的未知异常，然后对返回信息进行
 * 一定的修改，对用户更加友好
 *
 * @ControllerAdvice 提供了多种指定Advice规则的定义方式，默认什么都不写，则是Advice所有Controller，当然你也可以通过下列的方式指定规则
 * 比如对于 String[] value() default {} , 写成@ControllerAdvice("org.my.pkg") 或者 @ControllerAdvice(basePackages="org.my.pkg"),
 * 则匹配org.my.pkg包及其子包下的所有Controller，当然也可以用数组的形式指定，
 * 如：@ControllerAdvice(basePackages={"org.my.pkg", "org.my.other.pkg"}),
 * 也可以通过指定注解来匹配，比如我自定了一个 @CustomAnnotation 注解，我想匹配所有被这个注解修饰的 Controller，
 * 可以这么写：@ControllerAdvice（annotations={CustomAnnotation.class})
 * @ExceptionHandler 注解中可以添加参数，参数是某个异常类的class，代表这个方法专门处理该类异常，
 * 比如这样：@ExceptionHandler(NumberFormatException.class)
 * @ControllerAdvice与@ExceptionHandler配合使用可实现全局异常处理。
 */
@Slf4j
@RestControllerAdvice
@Order(1)
public class GlobalExceptionHandler {

    @ExceptionHandler(value = Exception.class)
    @ResponseBody
    private ServerResponse exceptionHandler(Exception ex, HttpServletRequest request) {
        Integer languageKey = parseLk();
        if (ex instanceof IllegalArgumentException) {
            log.error(ReliaqubeProgramExceptionEnum.IllegalArgumentException.getName(), ex);
            // 此处在weibull这个项目多为从接口处获取的计算结果不对无法解析导致的，所以可以给到一个无法计算的提示
            // 若不是无法计算的IllegalArgumentException都在这儿统一返回相关信息
            return ServerResponse.builder().status(ReliaqubeProgramExceptionEnum.IllegalArgumentException.getIndex()).message(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ex instanceof NumberFormatException ? ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析 : ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, languageKey)).build();
        } else if (ex instanceof IndexOutOfBoundsException) {
            // 处理数组下标越界的异常，该异常多是因为数据出现问题，需要后台查询原因
            log.error(ReliaqubeProgramExceptionEnum.IndexOutOfBoundsException.getName(), ex);
            return ServerResponse.builder().status(ReliaqubeProgramExceptionEnum.IndexOutOfBoundsException.getIndex()).message(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.未知错误, languageKey)).build();
        } else if (ex instanceof NullPointerException || ex.getMessage() == null) {
            // 处理空指针的异常，该异常多是因为数据出现问题，需要后台查询原因
            log.error(ReliaqubeProgramExceptionEnum.NullPointerException.getName(), ex);
            return ServerResponse.builder().status(ReliaqubeProgramExceptionEnum.NullPointerException.getIndex()).message(ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.未知错误, languageKey)).build();
        } else if (ex instanceof DescribeException) {
            // 处理从程序各处抛出的自定义异常，该异常均为程序各个地方手动抛出的异常
            DescribeException e = (DescribeException) ex;
            log.info(ReliaqubeProgramExceptionEnum.DescribeException.getName(), ex);
            return ServerResponse.builder().status(e.getStatus()).message(e.getMessage()).build();
        } else if (ex instanceof DescribeException) {
            // 处理从程序各处抛出的自定义异常，该异常均为程序各个地方手动抛出的异常
            DescribeException e = (DescribeException) ex;
            log.info(ReliaqubeProgramExceptionEnum.DescribeException.getName(), ex);
            return ServerResponse.builder().status(e.getStatus()).message(e.getMessage()).build();
        } else if (ex instanceof MethodArgumentNotValidException) {
            // 处理注解标注的字段的检验的多语言问题
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.getLanguageMessageByCode(((MethodArgumentNotValidException) ex).getBindingResult().getAllErrors().get(0).getDefaultMessage(), languageKey));
        } else if (ex instanceof ConstraintViolationException) {
            // 处理直接写在接口上的注解数据验证不通过的多语言问题，主要是@RequestParam需要的必传参数传入空数据引发的错误
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.getLanguageMessageByCode(((ConstraintViolationException) ex).getConstraintViolations().iterator().next().getMessageTemplate(), languageKey));
        } else if (ex instanceof MissingServletRequestParameterException) {
            // 处理必传参数未传的错误，主要是@RequestParam需要的必传参数未传入数据引发的错误
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
        } else if (ex instanceof HttpMessageNotReadableException) {
            // HttpMessageNotReadableException ： Http消息不可读异常
            // 多数情况是在http controller 层 传入参数时所造成的不可解析的异常
            // 如"1.2" 转 Integer失败 "1.2.2" 转 Double 失败，报此异常
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.请检查数据, languageKey);
        } else {
            // 上述都没有拦截到的异常，记录异常信息，返回未知错误
            log.error(ReliaqubeProgramExceptionEnum.SysCustomException.getName(), ex);
            return ServerResponse.createByErrorCodeMessage(ReliaqubeProgramExceptionEnum.SysCustomException.getIndex(), ReliaqubeParamDataCheckEnum.getLanguageMessageByKey(ReliaqubeParamDataCheckEnum.未知错误, languageKey));
        }
    }

    /**
     * 解析用户信息
     *
     * @return
     */
    public static HeaderUserInfo parseUserInfo() {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String language = LocaleContextHolder.getLocale().getLanguage();
        HeaderUserInfo userInfo = new HeaderUserInfo();
        userInfo.setUserId(user.getId());
        userInfo.setUserName(user.getUsername());
        // 此处当前仅适配中英文，后续有其他语言再进行增加适配
        userInfo.setLanguageKey(language.equals("zh") ? ReliaqubeLanguageKeyEnum.中文.getIndex() : ReliaqubeLanguageKeyEnum.英语.getIndex());
        return userInfo;
    }

    public static Integer parseLk() {
        return parseUserInfo().getLanguageKey();
    }
}
