package com.nationrel.modules.rga.controller;

import com.nationrel.modules.rga.entity.RgaAnalysis;
import com.nationrel.modules.rga.general.common.ServerResponse;
import com.nationrel.modules.rga.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.nationrel.modules.rga.service.RgaAnalysisService;

import java.util.List;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import com.nationrel.modules.rga.handler.GlobalExceptionHandler;

/**
 * @author hcs on 2021/5/24.
 */
@Tag(name = "加速数据分析-分析部分")
@RestController
@RequestMapping("/rga/analysis")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaAnalysisController {

    private final RgaAnalysisService analysisService;

    @Operation(summary = "试验数据")
    @GetMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "altaId", description = "分析Id", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<RgaAnalysis>> analysisList(HttpServletRequest request,
                                                          @RequestParam(required = false) String altaId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.listByPage(userInfo, altaId);
    }

    @Operation(summary = "分析创建")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> createAnalysis(HttpServletRequest request, @Valid @RequestBody RgaAnalysis entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.addAnalysis(userInfo, entity);
    }

    @Operation(summary = "分析修改")
    @PutMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateAnalysis(HttpServletRequest request, @Valid @RequestBody RgaAnalysis entity) {
        if (StringUtils.isBlank(entity.getEndTimeName())) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.请输入正确的寿命单位, GlobalExceptionHandler.parseLk());
        }
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.updateAnalysis(userInfo, entity);
    }

    @Operation(summary = "分析名称修改")
    @PostMapping("/name")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> updateAnalysisName(HttpServletRequest request, @RequestBody RgaAnalysis entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return analysisService.updateAnalysisName(userInfo, entity);
    }

    @Operation(summary = "分析删除")
    @DeleteMapping("/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> deleteAnalysis(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        return analysisService.deleteAnalysis(analysisId);
    }

    @Operation(summary = "分析复制")
    @PostMapping("/copy/{analysisId}")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> copyAnalysis(HttpServletRequest request, @PathVariable(value = "analysisId") String analysisId) {
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, GlobalExceptionHandler.parseLk());
        }
        return analysisService.copyAnalysis(analysisId);
    }

    @Operation(summary = "导入模板下载")
    @GetMapping("/import-template")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "分析Id", in = ParameterIn.QUERY)
    })
    public void importTemplate(HttpServletResponse response, @RequestParam(required = false) String analysisId) {
        analysisService.importTemplate(response, analysisId);
    }

    @Operation(summary = "导入")
    @PostMapping("/importData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse importData(HttpServletRequest request,
                                     @RequestParam("file") MultipartFile file,
                                     @RequestParam("altaId") String altaId,
                                     @RequestParam("analysisId") String analysisId) throws Exception{
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        analysisService.importData(userInfo, file, altaId, analysisId);
        return ServerResponse.createBySuccess();
    }

    @Operation(summary = "导出")
    @GetMapping("/exportData")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public void exportData(HttpServletResponse response, @RequestParam("analysisId") String analysisId) {
        analysisService.exportData(response, analysisId);
    }
}
