package com.nationrel.modules.rga.controller;

import com.nationrel.modules.rga.dto.RgaAnalysisDataSaveDto;
import com.nationrel.modules.rga.dto.RgaCalDataDto;
import com.nationrel.modules.rga.dto.RgaFastCalculateDto;
import com.nationrel.modules.rga.dto.RgaImageDetailDto;
import com.nationrel.modules.rga.entity.RgaSetting;
import com.nationrel.modules.rga.handler.GlobalExceptionHandler;
import com.nationrel.modules.rga.vo.*;
import com.nationrel.modules.rga.service.RgaCalcResultService;
import com.nationrel.modules.rga.service.RgaColumnAndDataService;
import com.nationrel.modules.rga.service.RgaPlotService;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import com.nationrel.modules.rga.general.common.ServerResponse;
import com.nationrel.modules.rga.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author Aimee on 2020/10/19.
 */
@Tag(name = "寿命数据分析-数据部分")
@RestController
@RequestMapping("/rga/data")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaAnalysisDataController {

    private final RgaColumnAndDataService columnAndDataService;
    private final RgaCalcResultService calcResultService;
    private final RgaPlotService plotService;

    @Operation(summary = "获取数据")
    @GetMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<RgaAnalysisSetDataVo> getAnalysisData(
            @RequestParam String analysisId,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "1000000000") int pageSize) {
        return columnAndDataService.listDataByPage(analysisId, pageIndex, pageSize);
    }

    @Operation(summary = "获取计算结果数据")
    @GetMapping("/calcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "analysisId", description = "数据ID", in = ParameterIn.QUERY)
    })
    public ServerResponse<List<RgaCalcResultVo>> getCalcResult(@RequestParam String analysisId) {
        return columnAndDataService.getCalcResult(analysisId);
    }

    @Operation(summary = "保存数据")
    @PostMapping("")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<String> saveData(HttpServletRequest request, @RequestBody RgaAnalysisDataSaveDto entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        // 数据校验
        return columnAndDataService.updateData(userInfo, entity);
    }

    @Operation(summary = "计算数据")
    @PostMapping("/calcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<RgaCalcResultsVo> calcResult(HttpServletRequest request, @RequestBody RgaCalDataDto entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        // 数据校验
        return calcResultService.calcResult(userInfo, entity);
    }

    @Operation(summary = "快速计算数据")
    @PostMapping("/fastCalcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<RgaAnalysisCalcVo> fastCalcResult(HttpServletRequest request, @RequestBody RgaFastCalculateDto entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return calcResultService.fastCalcResult(userInfo, entity);
    }

    @Operation(summary = "获取计算器设置")
    @GetMapping("/historyCalcResult")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER),
            @Parameter(name = "settingId", description = "设置ID", in = ParameterIn.QUERY)
    })
    public ServerResponse calcResult(HttpServletRequest request, @RequestParam String analysisId) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        if (StringUtils.isBlank(analysisId)) {
            return ServerResponse.createByErrorMessage(ReliaqubeParamDataCheckEnum.分析ID不可以为空, userInfo.getLanguageKey());
        }
        return  ServerResponse.createBySuccess(calcResultService.calcResult(analysisId));
    }

    @Operation(summary = "图像信息")
    @PostMapping("/image")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse<RgaPlotVo> fastCalcResult(HttpServletRequest request, @RequestBody RgaImageDetailDto entity) {
        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
        return plotService.selectEchartDetail(userInfo, entity);
    }

    @Operation(summary = "更新画图的标记以及xy区间信息/设置信息")
    @PostMapping("/setting")
    @Parameters(value = {
            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
    })
    public ServerResponse updateSetting(HttpServletRequest request, @RequestBody RgaSetting param) {
        return plotService.updateSetting(param, GlobalExceptionHandler.parseLk());
    }

//    @Operation(summary = "导入")
//    @PostMapping("/importData")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
//    public ServerResponse importData(HttpServletRequest request,
//                                     @RequestParam("file") MultipartFile file,
//                                     @RequestParam("dataId") String dataId,
//                                     @RequestParam("settingId") String settingId,
//                                     @RequestParam("analysisId") String analysisId) throws IOException {
//        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
//        ewbAnalysisDataDetailService.importData(userInfo, file, dataId, settingId, analysisId);
//        return ServerResponse.createBySuccess();
//    }
//
//    @Operation(summary = "导出")
//    @GetMapping("/exportData")
//    @Parameters(value = {
//            @Parameter(name = "Authorization", description = "验证信息", in = ParameterIn.HEADER)
//    })
//    public void exportData(HttpServletRequest request, HttpServletResponse response,
//                                     @RequestParam("dataId") String dataId) throws IOException {
//        HeaderUserInfo userInfo = GlobalExceptionHandler.parseUserInfo();
//        ewbAnalysisDataDetailService.exportData(response, userInfo, dataId);
//    }
//


}
