package com.nationrel.modules.rga.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.rga.entity.Student;
import com.nationrel.modules.rga.service.IStudentService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.aspect.annotation.PermissionData;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.Arrays;

/**
 * @Description: Student
 * @Author: nationrel
 * @Date: 2024-01-24
 * @Version: V1.0
 *
 */
@Tag(name = "Student")
@RestController
@RequestMapping("/example/student")
@Slf4j
public class StudentController extends JeecgController<Student, IStudentService> {
    @Autowired
    private IStudentService studentService;

    /**
     * 分页列表查询
     *
     * @param student
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    //@AutoLog(value = "Student-分页列表查询")
    @Operation(summary = "Student-分页列表查询")
    @PermissionData(pageComponent = "example/StudentList")
    @PostMapping(value = "/list")
    public Result<IPageExtField<Student>> queryPageList(@RequestBody Student student,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        QueryWrapper<Student> queryWrapper = QueryGenerator.initQueryWrapper(student, req.getParameterMap());
        IPageExtField<Student> page = new PageExtField<>(Student.class, pageNo, pageSize);
        IPageExtField<Student> pageList = studentService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param student
     * @return
     */
    @AutoLog(value = "Student-添加")
    @Operation(summary = "Student-添加")
    @RequiresPermissions("demo:student:add")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody Student student) {
        studentService.save(student);
        return Result.OK(MessageUtils.get("nationrel_add_success"));
    }

    /**
     * 编辑
     *
     * @param student
     * @return
     */
    @AutoLog(value = "Student-编辑")
    @Operation(summary = "Student-编辑")
    @RequiresPermissions("demo:student:edit")
    @RequestMapping(value = "/edit", method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody Student student) {
        studentService.updateById(student);
        return Result.OK(MessageUtils.get("nationrel_edit_success"));
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "Student-通过id删除")
    @Operation(summary = "Student-通过id删除")
    @RequiresPermissions("demo:student:delete")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        studentService.removeById(id);
        return Result.OK(MessageUtils.get("nationrel_delete_success"));
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "Student-批量删除")
    @Operation(summary = "Student-批量删除")
    @RequiresPermissions("demo:student:deleteBatch")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.studentService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK(MessageUtils.get("nationrel_delete_batch_success"));
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    //@AutoLog(value = "Student-通过id查询")
    @Operation(summary = "Student-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Student> queryById(@RequestParam(name = "id", required = true) String id) {
        Student student = studentService.getById(id);
        if (student == null) {
            return Result.error(MessageUtils.get("nationrel_no_found_data"));
        }
        return Result.OK(student);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param student
     */
    @RequiresPermissions("demo:student:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, Student student) {
        return super.exportXls(request, student, Student.class, "Student");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequiresPermissions("demo:student:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, Student.class);
    }

}
