// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.rga.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.entity.RgaAnalysis;
import com.nationrel.modules.rga.entity.RgaProject;
import com.nationrel.modules.rga.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.rga.general.domain.enums.common.AuthorityProgramExceptionEnum;
import com.nationrel.modules.rga.general.util.common.BasicDataUtil;
import com.nationrel.modules.rga.mapper.RgaProjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

/**
 * @ClassName RgaProjectService
 * @Description 加速数据分析数据的操作
 * @Author hcs
 * @CreateDate 2021-06-26
 * @Version V1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaProjectService {

    private final RgaProjectMapper projectMapper;
    private final RgaAnalysisService analysisService;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param altaName
     * @param startDate
     * @param endDate
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public Result<IPage<RgaProject>> listByPage(HeaderUserInfo userInfo, String altaId, String altaName, Date startDate, Date endDate, int pageIndex, int pageSize) {
        // 创建分页条件
        Page<RgaProject> page = new Page<>(pageIndex, pageSize);
        // 创建基础查询条件
        LambdaQueryWrapper<RgaProject> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RgaProject::getCreateBy, userInfo.getUserName());
        queryWrapper.eq(RgaProject::getDelFlag, RgaConstants.DEFAULT_DEL_FLAG);
        if (StringUtils.isNotBlank(altaId)) {
            queryWrapper.like(RgaProject::getProjectId, altaId);
        }
        if (StringUtils.isNotBlank(altaName)) {
            queryWrapper.like(RgaProject::getProjectName, altaName);
        }
        if (startDate != null) {
            queryWrapper.ge(RgaProject::getCreateTime, startDate);
        }
        if (endDate != null) {
            queryWrapper.le(RgaProject::getCreateTime, endDate);
        }
        queryWrapper.orderByDesc(RgaProject::getUpdateTime);
        return Result.ok(projectMapper.selectPage(page, queryWrapper));
    }

    /**
     * 添加分析数据信息
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Result<String> addAnalysis(HeaderUserInfo userInfo, RgaProject entity) {

        String altaId = BasicDataUtil.uuidGenerator();
        entity.setProjectId(altaId);
        entity.setCreateTime(new Date());
        entity.setUpdateTime(new Date());
        entity.setCreateBy(userInfo.getUserName());
        entity.setUpdateBy(userInfo.getUserName());
        entity.setDelFlag(RgaConstants.DEFAULT_DEL_FLAG);
        projectMapper.insert(entity);
        // 添加分析数据
        RgaAnalysis analysis = new RgaAnalysis();
        analysis.setProjectId(altaId);
        analysis.setAnalysisName("rga-a");
        analysisService.addAnalysis(userInfo, analysis);

        return Result.ok();
    }

    /**
     * 更新修改者信息
     * @param userInfo
     * @param altaId
     */
    public static void updateUpdateInfo(HeaderUserInfo userInfo, String altaId, RgaProjectMapper projectMapper) {
        projectMapper.updateUpdateInfo(new Date(), userInfo.getUserName(),altaId);
    }

    /**
     * 更新数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public Result<String> updateProject(HeaderUserInfo userInfo, RgaProject entity) {
        if (StringUtils.isBlank(entity.getProjectId()) || StringUtils.isBlank(entity.getProjectId())) {
            return Result.error(AuthorityProgramExceptionEnum.getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        projectMapper.updateById(entity);
        updateUpdateInfo(userInfo, entity.getProjectId(), projectMapper);
        return Result.ok();
    }

    /**
     * 逻辑删除数据
     *
     * @param userInfo
     * @param altaId
     * @return
     * @throws Exception
     */
    public Result<String> deleteProject(HeaderUserInfo userInfo, String altaId) {
        RgaProject analysis = projectMapper.selectById(altaId);
        if (Objects.equals(userInfo.getUserName(), analysis.getCreateBy())) {
            analysis.setDelFlag(RgaConstants.DEL_FLAG);
            projectMapper.updateById(analysis);
        }
        return Result.ok();
    }
}

