package com.nationrel.modules.rga.service;

import com.nationrel.modules.rga.constant.RgaConstants;
import com.nationrel.modules.rga.entity.RgaData;
import com.nationrel.modules.rga.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.rga.general.exception.DescribeException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author Aimee on 2020/10/28.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class RgaValidatorService {


    /**
     * @param dataDetailsModel     分析明细数据
     * @param groupingFlag 页面复选框"分组数据"选中标记
     * @param leftFlag     页面复选框"包含区间删失数据或左删失数据"选中
     * @param rightFlag    页面复选框"包含右删失数据"选中标记标记
     * @Title: validatorAnalysisDetailData
     * @Description: 分析明细数据输入校验
     * 消息处理辅助类
     */
    public void validatorAnalysisDetailData(List<RgaData> dataDetailsModel, String groupingFlag, String leftFlag, String rightFlag, int startDataNo, Integer languageKey) {
        // 当前页数据总数量
        if (CollectionUtils.isEmpty(dataDetailsModel)) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.暂未添加数据, languageKey);
        }
        // 判断页面明细数据列表是否有值
        // 行号
        int rowNum = 0;
        if (startDataNo > 0) {
            rowNum = startDataNo - 1;
        }
        rowNum = rowNum % 10;

        for (RgaData dataDetailModel : dataDetailsModel) {
            if (dataDetailModel.getFailureEndTime() != null) {
                RgaData data = new RgaData();
                rowNum++;
                // 获取数量
                Integer quantity = dataDetailModel.getQuantity();
                // 页面复选框"分组数据"选中
                if (RgaConstants.FLAG_CHECKED.equals(groupingFlag)) {
                    // 数量校验
                    this.validatorQuantity(rowNum, quantity, languageKey);
                }
                // 获取最近检查时间
                Double lastInspectTime = dataDetailModel.getLastInspectTime();
                // 页面复选框"包含区间删失数据或左删失数据"选中
                if (RgaConstants.FLAG_CHECKED.equals(leftFlag)) {
                    // 截取指定小数位数的小数
                    lastInspectTime = interceptLastInspectTimeForAnalysis(data, lastInspectTime);
                    // 最近检查时间校验
                    this.validatorLastInspectTime(rowNum, lastInspectTime, languageKey);
                }

                // 取得F或者S的状态
                String failureSuspension = dataDetailModel.getFailureSuspension();
                // 页面复选框"包含右删失数据"选中
                if (RgaConstants.FLAG_CHECKED.equals(rightFlag)) {
                    // F或者S的状态校验
                    this.validatorFs(rowNum, failureSuspension, languageKey);
                }

                // 获取结束时间
                Double failureEndTime = dataDetailModel.getFailureEndTime();
                // 截取指定小数位数的小数
                failureEndTime = interceptLastInspectTimeForAnalysis(data, failureEndTime);
                // 结束时间校验
                this.validatorFailureEndTime(rowNum, failureEndTime, lastInspectTime, leftFlag, languageKey);
            }
        }
    }

    /**
     * @param rowNum          行数
     * @param failureEndTime  结束时间
     * @param lastInspectTime 最近检查时间
     * @param leftFlag        页面复选框"包含区间删失数据或左删失数据"选中
     * @Title: validatorFailureEndTime
     * @Description: 结束时间校验
     * 消息处理辅助类
     */
    public void validatorFailureEndTime(int rowNum, Double failureEndTime, Double lastInspectTime, String leftFlag, Integer languageKey) throws DescribeException {
        // 结束时间不能为空
        if (failureEndTime == null) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间不可为空, languageKey);
        }else if (failureEndTime < 0.0001 || failureEndTime > 999999999.9999) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间范围必须是999999999, languageKey);
        }

        if (RgaConstants.FLAG_CHECKED.equals(leftFlag) && lastInspectTime != null) {
            BigDecimal valF = parseBigDecimal(failureEndTime);
            BigDecimal valL = parseBigDecimal(lastInspectTime);
            // 校验结束时间必须大于最近检查时间
            if (valF.compareTo(valL) < 0) {
                throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.结束时间必须大于等于最近检查时间, languageKey);
            }
        }
    }

    /**
     * @param rowNum            行数
     * @param failureSuspension F或者S的状态
     * @Title: validatorFs
     * @Description: F或者S的状态校验
     * 消息处理辅助类
     */
    private void validatorFs(int rowNum, String failureSuspension, Integer languageKey) throws DescribeException {
        // F或者S不能为空
        if (StringUtils.isEmpty(failureSuspension)) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.F或者S不可为空, languageKey);
        }
        // 输入必须是f或者s
        if (StringUtils.isNotEmpty(failureSuspension) && !(RgaConstants.STATE_SUSPENSION.equalsIgnoreCase(failureSuspension) || RgaConstants.STATE_FUFPENSION.equalsIgnoreCase(failureSuspension))) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.输入内容必须是F或者S, languageKey);
        }
    }

    /**
     * @param rowNum   行数
     * @param quantity 数量
     * @Title: validatorQuantity
     * @Description: 数量校验
     * 消息处理辅助类
     */
    private void validatorQuantity(int rowNum, Integer quantity, Integer languageKey) throws DescribeException {
        // 数量不能为空
        if (quantity == null) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.数量不可为空, languageKey);
        } else if (parseInteger(quantity) < 1 || parseInteger(quantity) > 99999) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.数量范围必须是99999, languageKey);
        }
    }

    /**
     * @param rowNum          行数
     * @param lastInspectTime 最近检查时间
     * @Title: validatorLastInspectTime
     * @Description: 最近检查时间校验
     * 消息处理辅助类
     */
    private void validatorLastInspectTime(int rowNum, Double lastInspectTime, Integer languageKey) throws DescribeException {
        if (lastInspectTime == null) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.最近检查时间不可为空, languageKey);
        } else if (lastInspectTime < 0 || lastInspectTime > 99999) {
            throw new DescribeException(rowNum, ReliaqubeParamDataCheckEnum.最近检查时间范围必须大于0, languageKey);
        }
    }

    /**
     * @param data            转换后的对象
     * @param lastInspectTime 最近检查时间
     * @return 截取之后的小数
     * @Title: interceptLastInspectTimeForAnalysis
     * @Description: 截取最近检查时间的小数位数
     */
    private Double interceptLastInspectTimeForAnalysis(RgaData data, Double lastInspectTime) {
        if (lastInspectTime != null && !isMatch(lastInspectTime, "^-?(\\d*|\\d*\\.\\d{1," + RgaConstants.NUM_DIGIT_4 + "})$")) {
            // 截取指定小数位数
            double newLastInspectTime = round(lastInspectTime, RgaConstants.NUM_DIGIT_4);
            // 赋值
            data.setLastInspectTime(newLastInspectTime);
        }
        return lastInspectTime;
    }


    /**
     * @param val 验证字符串
     * @return 验证结果(true : 匹配, false : 不匹配)
     * @Title: isInteger
     * @Description: 匹配整数 ^-?[1-9]\\d*$
     */
    public static boolean isInteger(String val) {
        return val != null && val.matches("^-?([1-9]\\d*|0)$");
    }

    /**
     * @param val 验证字符串
     * @return 验证结果(true : 匹配, false : 不匹配)
     * @Title: isNumber
     * @Description: 数值
     */
    public static boolean isNumber(String val) {
        return isFloat(val) || isInteger(val);
    }

    /**
     * @param val 验证字符串
     * @return 验证结果(true : 匹配, false : 不匹配)
     * @Title: isFloat
     * @Description: 匹配浮点数 ^-?([1-9]\\d*.\\d*|0.\\d*[1-9]\\d*|0?.0+|0)$
     */
    public static boolean isFloat(String val) {
        return val != null && val
                .matches("^-?([1-9]\\d*\\.\\d+|0\\.\\d+[1-9]\\d*|0?\\.\\d+|0?\\.0+|0)$");
    }

    /**
     * @param obj 待转换参数
     * @return Integer
     * @Title:
     * @Description: 对象转换为整型
     */
    public static Integer parseInteger(Object obj) {
        if (obj == null || "".equals(obj)) {
            return null;
        } else {
            return Integer.valueOf(String.valueOf(obj));
        }
    }

    /**
     * Object转换为BigDecimal
     *
     * @param objIn Object
     * @return 处理后的BigDecimal
     */
    public static BigDecimal parseBigDecimal(Object objIn) {
        if (objIn == null) {
            return null;
        } else {
            return new BigDecimal(String.valueOf(objIn));
        }
    }

    /**
     * @param extRegex
     * @return boolean
     * @Title: isMatch
     * @Description: 根据指定的正则表达式与指定字符串做匹配
     */
    public static boolean isMatch(Double val, String extRegex) {
        String tmpReg = extRegex;
        if (StringUtils.isEmpty(extRegex)) {
            tmpReg = "^[A-Za-z]+[A-Za-z0-9._]*$";
        }
        return val != null && val.toString().matches(tmpReg);
    }

    /**
     * @param dbln     待处理数字
     * @param radixbit 小数位数
     * @return 四舍五入的结果
     * @Title: round
     * @Description: 双精度四舍五入处理
     */
    public static Double round(Double dbln, int radixbit) {
        if (dbln == null) {
            return null;
        }
        BigDecimal bdc = new BigDecimal(String.valueOf(dbln));
        return bdc.setScale(radixbit, RoundingMode.HALF_UP).doubleValue();
    }
}
